class AuthenticationException(Exception):
    pass


class InvalidRequestException(Exception):
    pass


class OperationNotSupportedException(Exception):
    pass


class ProfileNotSupportedException(Exception):
    pass


class HandleValueNotFoundException(Exception):
    pass